/******************************************************************************
* In order to make life a little bit easier when using the GIF file format,   *
* this library was written, and which does all the dirty work...	      *
*									      *
*					Written by Gershon Elber,  Jun. 1989  *
*					Hacks by Eric S. Raymond,  Sep. 1992  *
*******************************************************************************
* History:								      *
* 14 Jun 89 - Version 1.0 by Gershon Elber.				      *
*  3 Sep 90 - Version 1.1 by Gershon Elber (Support for Gif89, Unique names). *
* 15 Sep 90 - Version 2.0 by Eric S. Raymond (Changes to suoport GIF slurp)   *
* 26 Jun 96 - Version 3.0 by Eric S. Raymond (Full GIF89 support)             *
* 19 Jan 98 - Version 3.1 by Jareth Hein (Support for user-defined I/O).      *
******************************************************************************/

#ifndef GIF_LIB_H
#define GIF_LIB_H

#ifdef FPTEX
#include <win32lib.h>
#endif

#ifndef _MSC_VER
#define __cdecl
#endif

#if defined(WIN32)
# undef DrawText
# ifdef GIF_DLL
#  ifdef MAKE_GIF_DLL
#   define GIFDLL __declspec( dllexport)
#  else
#   define GIFDLL __declspec( dllimport)
#  endif
# else
#  define GIFDLL
# endif
#else /* ! WIN32 */
# define GIFDLL
#endif

#define GIF_LIB_VERSION	" Version 3.1, "

#define	GIF_ERROR	0
#define GIF_OK		1

#ifndef TRUE
#define TRUE		1
#define FALSE		0
#endif

#ifndef NULL
#define NULL		0
#endif /* NULL */

#define GIF_FILE_BUFFER_SIZE 16384  /* Files uses bigger buffers than usual. */

typedef	int		GifBooleanType;
typedef	unsigned char	GifPixelType;
typedef unsigned char *	GifRowType;
typedef unsigned char	GifByteType;

#ifdef SYSV
#define VoidPtr char *
#else
#define VoidPtr void *
#endif /* SYSV */

typedef struct GifColorType {
    GifByteType Red, Green, Blue;
} GifColorType;

typedef struct ColorMapObject
{
    int	ColorCount;
    int BitsPerPixel;
    GifColorType *Colors;		/* on malloc(3) heap */
}
ColorMapObject;

typedef struct GifImageDesc {
    int Left, Top, Width, Height,	/* Current image dimensions. */
	Interlace;			/* Sequential/Interlaced lines. */
    ColorMapObject *ColorMap;		/* The local color map */
} GifImageDesc;

/* I/O operations.  If you roll your own, they need to be semantically equivilent to
   fread/fwrite, with an additional paramater to hold data local to your method. */
typedef size_t (__cdecl *Gif_rw_func)(GifByteType *buffer, size_t size, VoidPtr method_data);
/* Finish up stream. Non-zero return indicates failure */
typedef int (__cdecl *Gif_close_func)(VoidPtr close_data);
/* Error handling function */
typedef void (__cdecl *Gif_error_func)(const char *string, VoidPtr error_data);

typedef struct GifFileType {
    int SWidth, SHeight,		/* Screen dimensions. */
	SColorResolution, 		/* How many colors can we generate? */
	SBackGroundColor;		/* I hope you understand this one... */
    ColorMapObject *SColorMap;		/* NULL if it doesn't exist. */
    int ImageCount;			/* Number of current image */
    GifImageDesc Image;			/* Block describing current image */
    struct SavedImage *SavedImages;	/* Use this to accumulate file state */
    VoidPtr Private;	  		/* Don't mess with this! */
    VoidPtr GifIO;			/* Contains all information for I/O */
} GifFileType;

typedef enum {
    UNDEFINED_RECORD_TYPE,
    SCREEN_DESC_RECORD_TYPE,
    IMAGE_DESC_RECORD_TYPE,		/* Begin with ',' */
    EXTENSION_RECORD_TYPE,		/* Begin with '!' */
    TERMINATE_RECORD_TYPE		/* Begin with ';' */
} GifRecordType;

/* DumpScreen2Gif routine constants identify type of window/screen to dump.  */
/* Note all values below 1000 are reserved for the IBMPC different display   */
/* devices (it has many!) and are compatible with the numbering TC2.0        */
/* (Turbo C 2.0 compiler for IBM PC) gives to these devices.		     */
typedef enum {
    GIF_DUMP_SGI_WINDOW = 1000,
    GIF_DUMP_X_WINDOW = 1001
} GifScreenDumpType;

/******************************************************************************
*  GIF89 extension function codes                                             *
******************************************************************************/

#define COMMENT_EXT_FUNC_CODE		0xfe	/* comment */
#define GRAPHICS_EXT_FUNC_CODE		0xf9	/* graphics control */
#define PLAINTEXT_EXT_FUNC_CODE		0x01	/* plaintext */
#define APPLICATION_EXT_FUNC_CODE	0xff	/* application block */

/******************************************************************************
* IO related routines.  Defined in gif_io.c                                   *
******************************************************************************/
GIFDLL GifFileType *GifSetup(void);
GIFDLL void GifFree(GifFileType *GifFile);
GIFDLL void GifSetReadFunc (GifFileType *GifFile, Gif_rw_func func, VoidPtr data);
GIFDLL void GifSetWriteFunc(GifFileType *GifFile, Gif_rw_func func, VoidPtr data);
GIFDLL void GifSetCloseFunc(GifFileType *GifFile, Gif_close_func func, VoidPtr data);

/******************************************************************************
* O.K., here are the routines one can access in order to encode GIF file:     *
* (GIF_LIB file EGIF_LIB.C).						      *
******************************************************************************/

GIFDLL void EGifOpenFileName(GifFileType *GifFile, char *GifFileName, int GifTestExistance);
GIFDLL void EGifOpenFileHandle(GifFileType *GifFile, int GifFileHandle);
GIFDLL void EGifInitWrite(GifFileType *GifFile);
GIFDLL int EGifSpew(GifFileType *GifFile);
GIFDLL void EGifSetGifVersion(char *Version);
GIFDLL void EGifPutScreenDesc(GifFileType *GifFile,
	int GifWidth, int GifHeight, int GifColorRes, int GifBackGround,
	ColorMapObject *GifColorMap);
GIFDLL void EGifPutImageDesc(GifFileType *GifFile,
	int GifLeft, int GifTop, int Width, int GifHeight, int GifInterlace,
	ColorMapObject *GifColorMap);
GIFDLL void EGifPutLine(GifFileType *GifFile, GifPixelType *GifLine, int GifLineLen);
GIFDLL void EGifPutPixel(GifFileType *GifFile, GifPixelType GifPixel);
GIFDLL void EGifPutComment(GifFileType *GifFile, char *GifComment);
GIFDLL void EGifPutExtension(GifFileType *GifFile, int GifExtCode, int GifExtLen,
							VoidPtr GifExtension);
GIFDLL void EGifPutCode(GifFileType *GifFile, int GifCodeSize,
						   GifByteType *GifCodeBlock);
GIFDLL void EGifPutCodeNext(GifFileType *GifFile, GifByteType *GifCodeBlock);
GIFDLL int EGifCloseFile(GifFileType *GifFile);

#define	E_GIF_ERR_OPEN_FAILED	1		/* And EGif possible errors. */
#define	E_GIF_ERR_WRITE_FAILED	2
#define E_GIF_ERR_HAS_SCRN_DSCR	3
#define E_GIF_ERR_HAS_IMAG_DSCR	4
#define E_GIF_ERR_NO_COLOR_MAP	5
#define E_GIF_ERR_DATA_TOO_BIG	6
#define E_GIF_ERR_NOT_ENOUGH_MEM 7
#define GIF_ERR_NOT_ENOUGH_MEM 7
#define E_GIF_ERR_DISK_IS_FULL	8
#define E_GIF_ERR_CLOSE_FAILED	9
#define E_GIF_ERR_NOT_WRITEABLE	10

/******************************************************************************
* O.K., here are the routines one can access in order to decode GIF file:     *
* (GIF_LIB file DGIF_LIB.C).						      *
******************************************************************************/

GIFDLL void DGifOpenFileName(GifFileType *GifFile, const char *GifFileName);
GIFDLL void DGifOpenFileHandle(GifFileType *GifFile, int GifFileHandle);
GIFDLL void DGifInitRead(GifFileType *GifFile);
GIFDLL void DGifSlurp(GifFileType *GifFile);
GIFDLL void DGifGetScreenDesc(GifFileType *GifFile);
GIFDLL void DGifGetRecordType(GifFileType *GifFile, GifRecordType *GifType);
GIFDLL void DGifGetImageDesc(GifFileType *GifFile);
GIFDLL void DGifGetLine(GifFileType *GifFile, GifPixelType *GifLine, int GifLineLen);
GIFDLL void DGifGetPixel(GifFileType *GifFile, GifPixelType GifPixel);
GIFDLL void DGifGetComment(GifFileType *GifFile, char *GifComment);
GIFDLL void DGifGetExtension(GifFileType *GifFile, int *GifExtCode,
						GifByteType **GifExtension);
GIFDLL void DGifGetExtensionNext(GifFileType *GifFile, GifByteType **GifExtension);
GIFDLL void DGifGetCode(GifFileType *GifFile, int *GifCodeSize,
						GifByteType **GifCodeBlock);
GIFDLL void DGifGetCodeNext(GifFileType *GifFile, GifByteType **GifCodeBlock);
GIFDLL void DGifGetLZCodes(GifFileType *GifFile, int *GifCode);
GIFDLL int DGifCloseFile(GifFileType *GifFile);

#define	D_GIF_ERR_OPEN_FAILED	101		/* And DGif possible errors. */
#define	D_GIF_ERR_READ_FAILED	102
#define	D_GIF_ERR_NOT_GIF_FILE	103
#define D_GIF_ERR_NO_SCRN_DSCR	104
#define D_GIF_ERR_NO_IMAG_DSCR	105
#define D_GIF_ERR_NO_COLOR_MAP	106
#define D_GIF_ERR_WRONG_RECORD	107
#define D_GIF_ERR_DATA_TOO_BIG	108
#define D_GIF_ERR_NOT_ENOUGH_MEM 109
#define D_GIF_ERR_CLOSE_FAILED	110
#define D_GIF_ERR_NOT_READABLE	111
#define D_GIF_ERR_IMAGE_DEFECT	112
#define D_GIF_ERR_EOF_TOO_SOON	113

/******************************************************************************
* O.K., here are the routines from GIF_LIB file QUANTIZE.C.		      *
******************************************************************************/
GIFDLL int QuantizeBuffer(GifFileType *GifFile, unsigned int Width,
		   unsigned int Height, int *ColorMapSize,
		   GifByteType *RedInput, GifByteType *GreenInput,
		   GifByteType *BlueInput, GifByteType *OutputBuffer,
		   GifColorType *OutputColorMap);

/******************************************************************************
* O.K., here are the routines from GIF_LIB file GIF_ERR.C.		      *
******************************************************************************/
extern GIFDLL void GifSetErrorFunc(GifFileType *GifFile, Gif_error_func func, VoidPtr data);
extern GIFDLL void GifSetWarningFunc(GifFileType *GifFile, Gif_error_func func, VoidPtr data);
extern GIFDLL void GifInternError(GifFileType *GifFile, int errnum);
extern GIFDLL void GifInternWarning(GifFileType *GifFile, int errnum);
extern GIFDLL void GifError(GifFileType *GifFile, const char *err_str);
extern GIFDLL void GifWarning(GifFileType *GifFile, const char *err_str);

/******************************************************************************
* O.K., here are the routines from GIF_LIB file DEV2GIF.C.		      *
******************************************************************************/
extern GIFDLL int DumpScreen2Gif(char *FileName,
			  int ReqGraphDriver,
			  int ReqGraphMode1,
			  int ReqGraphMode2,
			  int ReqGraphMode3);

/*****************************************************************************
 *
 * Everything below this point is new after version 1.2, supporting `slurp
 * mode' for doing I/O in two big belts with all the image-bashing in core.
 *
 *****************************************************************************/

/******************************************************************************
* Color Map handling from ALLOCGIF.C					      *
******************************************************************************/

extern GIFDLL ColorMapObject *MakeMapObject(int ColorCount, GifColorType *ColorMap);
extern GIFDLL void FreeMapObject(ColorMapObject *Object);
extern GIFDLL ColorMapObject *UnionColorMap(ColorMapObject *ColorIn1,
			      ColorMapObject *ColorIn2,
			      GifPixelType ColorTransIn2[]);
extern GIFDLL int BitSize(int n);

/******************************************************************************
* Support for the in-core structures allocation (slurp mode).		      *
******************************************************************************/

/* This is the in-core version of an extension record */
typedef struct {
    int		ByteCount;
    char	*Bytes;		/* on malloc(3) heap */
} ExtensionBlock;

/* This holds an image header, its unpacked raster bits, and extensions */
typedef struct SavedImage {
    GifImageDesc	ImageDesc;

    char		*RasterBits;		/* on malloc(3) heap */

    int			Function;
    int			ExtensionBlockCount;
    ExtensionBlock	*ExtensionBlocks;	/* on malloc(3) heap */
} SavedImage;

extern GIFDLL void ApplyTranslation(SavedImage *Image, GifPixelType Translation[]);

extern GIFDLL void MakeExtension(SavedImage *New, int Function);
extern GIFDLL int AddExtensionBlock(SavedImage *New, int Length, GifByteType *data);
extern GIFDLL void FreeExtension(SavedImage *Image);

extern GIFDLL SavedImage *MakeSavedImage(GifFileType *GifFile, SavedImage *CopyFrom);
extern GIFDLL void FreeSavedImages(GifFileType *GifFile);

/******************************************************************************
* The library's internal utility font					      *
******************************************************************************/

#define GIF_FONT_WIDTH	8
#define GIF_FONT_HEIGHT	8
extern unsigned char AsciiTable[][GIF_FONT_WIDTH];

extern void DrawText(SavedImage *Image,
		     const int x, const int y,
		     const char *legend,
		     const int color);

extern void DrawBox(SavedImage *Image,
		     const int x, const int y,
		     const int w, const int d,
		     const int color);

void DrawRectangle(SavedImage *Image,
		     const int x, const int y,
		     const int w, const int d,
		     const int color);

extern void DrawBoxedText(SavedImage *Image,
		     const int x, const int y,
		     const char *legend,
		     const int border,
		     const int bg,
		     const int fg);

#endif /* GIF_LIB_H */
