/* System description file for Windows NT.
   Copyright (C) 1997, 1998 Free Software Foundation, Inc.

This file is part of Web2C.

Web2C is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Web2C is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Web2C; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef _GNU_W32_H_
#define _GNU_W32_H_

#pragma warning( disable : 4007 4096 4018 4244 )  

#if defined(WIN32)
#ifdef _DLL
#ifdef MAKE_GNUW32_DLL
#define GNUW32DLL __declspec( dllexport)
#else
#define GNUW32DLL __declspec( dllimport)
#endif
#else
#define GNUW32DLL
#endif
#else /* ! WIN32 */
#define GNUW32DLL
#endif

/*
 *      Define symbols to identify the version of Unix this is.
 *      Define all the symbols that apply correctly.
 */

#ifndef DOSISH
#define DOSISH
#endif

#ifndef MAXPATHLEN
#define MAXPATHLEN      _MAX_PATH
#endif

#define HAVE_DUP2       	1
#define HAVE_RENAME     	1
#define HAVE_RMDIR      	1
#define HAVE_MKDIR      	1
#define HAVE_GETHOSTNAME	1
#define HAVE_RANDOM		1
#define USE_UTIME		1
#define HAVE_MOUSE		1
#define HAVE_TZNAME		1

/* Define this so that winsock.h definitions don't get included when
   windows.h is...  For this to have proper effect, config.h must
   always be included before windows.h.  */ 
#if !defined(_WINSOCKAPI_) && !defined(WWW_WIN_DLL)
#define _WINSOCKAPI_    1
#endif

#include <windows.h>

/* Defines size_t and alloca ().  */
#include <malloc.h>

/* For proper declaration of environ.  */
#include <sys/types.h>
#include <fcntl.h>
#include <io.h>
#include <process.h>
#include <direct.h>
#include <stdio.h>
#include <stdlib.h>

#define NEW_GNUW32 0

struct stat {
        _dev_t st_dev;
        _ino_t st_ino;
        unsigned short st_mode;
        short st_nlink;
        short st_uid;
        short st_gid;
        _dev_t st_rdev;
        _off_t st_size;
        time_t st_atime;
        time_t st_mtime;
        time_t st_ctime;
        };

/* We want to compile with __STDC__ defined, but in this
   case, MSVC headers doe not define the standard names for
   the C library functions (_open() instead of open()) */
#include <oldnames.h>

#define getwd(dir)                 GetCurrentDirectory(MAXPATHLEN, dir)
#define index(s, c)                strchr(s, c)
#define pipe(fd, size, mode)       _pipe(fd, size, mode)
#define strcasecmp(s1, s2)         _stricmp(s1, s2)
#define strncasecmp(s1, s2, n)     _strnicmp(s1, s2, n)


/* Test for each symbol individually and define the ones necessary (some
   systems claiming Posix compatibility define some but not all). */

#if defined (S_IFBLK) && !defined (S_ISBLK)
#define        S_ISBLK(m)      (((m)&S_IFMT) == S_IFBLK)       /* block device */
#endif

#if defined (S_IFCHR) && !defined (S_ISCHR)
#define        S_ISCHR(m)      (((m)&S_IFMT) == S_IFCHR)       /* character device */
#endif

#if defined (S_IFDIR) && !defined (S_ISDIR)
#define        S_ISDIR(m)      (((m)&S_IFMT) == S_IFDIR)       /* directory */
#endif

#if defined (S_IFREG) && !defined (S_ISREG)
#define        S_ISREG(m)      (((m)&S_IFMT) == S_IFREG)       /* file */
#endif

#if defined (S_IFIFO) && !defined (S_ISFIFO)
#define        S_ISFIFO(m)     (((m)&S_IFMT) == S_IFIFO)       /* fifo - named pipe */
#endif

#if defined (S_IFLNK) && !defined (S_ISLNK)
#define        S_ISLNK(m)      (((m)&S_IFMT) == S_IFLNK)       /* symbolic link */
#endif

#if defined (S_IFSOCK) && !defined (S_ISSOCK)
#define        S_ISSOCK(m)     (((m)&S_IFMT) == S_IFSOCK)      /* socket */
#endif

/*
  Path separators.
  Redundant with <kpathsea/c-pathch.h>, but useful
  outside of kpathsea.
*/

/* On DOS, it's good to allow both \ and / between directories.  */
#ifndef IS_DIR_SEP
#define IS_DIR_SEP(ch) ((ch) == '/' || (ch) == '\\')
#endif
#ifndef IS_DEVICE_SEP
#define IS_DEVICE_SEP(ch) ((ch) == ':')
#endif
#ifndef NAME_BEGINS_WITH_DEVICE
#define NAME_BEGINS_WITH_DEVICE(name) (*(name) && IS_DEVICE_SEP((name)[1]))
#endif
/* On win32, UNC names are authorized */
#ifndef IS_UNC_NAME
#define IS_UNC_NAME(name) (strlen(name)>=3 && IS_DIR_SEP(*name)  \
                            && IS_DIR_SEP(*(name+1)) && isalpha(*(name+2)))
#endif

#ifndef min
#define min(a,b) ((b) > (a) ? (a) : (b))
#endif
#ifndef max
#define max(a,b) ((a) > (b) ? (a) : (b))
#endif

/* Functions from popen.c */

#define MAX_PIPES 128

BOOL look_for_cmd(const char *command, char **app_name, char **new_cmd);

extern GNUW32DLL FILE *popen(const char *, const char *);
extern GNUW32DLL int pclose(FILE *);

/* Functions from stat.c */

extern GNUW32DLL int __cdecl win32_stat(const char * path, struct stat * buf);

/* Functions from system.c */

extern GNUW32DLL int __cdecl win32_system(const char * cmd);

/* Functions from string.c */
/*
  Unfortunately, these functions are defined in kpathsea
  but useful elsewhere too. As they have a small footprint,
  they are duplicated for the moment and not exported.
*/
extern void *_xmalloc(unsigned size);
extern char *_xstrdup(const char *);
extern char *_concat(const char *s1, const char *s2);
extern char *_concat3(const char *s1, const char *s2, const char *s3);
extern char * __cdecl _concatn(const char *s1, ...);

/* Functions from volume.c */

typedef struct volume_info_data {
  struct volume_info_data * next;

  /* time when info was obtained */
  DWORD     timestamp;

  /* actual volume info */
  char *    root_dir;
  DWORD     serialnum;
  DWORD     maxcomp;
  DWORD     flags;
  char *    name;
  char *    type;
} volume_info_data;

extern int is_fat_volume (const char * name, const char ** pPath);
extern int get_volume_info (const char * name, const char ** pPath);

/* Functions from filename.c */

extern void dostounix_filename (char *p);
extern void unixtodos_filename (char *p);
extern int crlf_to_lf (int n, unsigned char *buf, unsigned *lf_count);
extern BOOL win32_get_long_filename (char * name, char * buf, int size);
extern const char *map_win32_filename (const char * name, const char ** pPath);
extern int _parse_root (char * name, char ** pPath);

/* Functions from user.c */
extern void init_user_info();

/* Functions from home.c */
extern char *get_home_directory (void);

/* Functions from time.h */
extern time_t convert_time (FILETIME ft);

#ifdef NEW_GNUW32

BOOL parse_cmdline(char *command, char **input, char **output, char *pipes[]);
char *build_cmdline(char **input, char **output, char *pipes[]);
#endif
/* ============================================================ */

#endif /* not _GNU_W32_H_ */
